//
// $Id: FileEffectiveRightsProbe.cpp,v 1.3 2006/09/06 17:31:23 bakerj Exp $
//
//****************************************************************************************//
// Copyright (c) 2006, The MITRE Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice, this list
//       of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright notice, this 
//       list of conditions and the following disclaimer in the documentation and/or other
//       materials provided with the distribution.
//     * Neither the name of The MITRE Corporation nor the names of its contributors may be
//       used to endorse or promote products derived from this software without specific 
//       prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
// SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
// OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//****************************************************************************************//

#include "FileEffectiveRightsProbe.h"

//****************************************************************************************//
//								FileEffectiveRightsProbe Class							  //	
//****************************************************************************************//
FileEffectiveRightsProbe* FileEffectiveRightsProbe::instance = NULL;

FileEffectiveRightsProbe::FileEffectiveRightsProbe() {
	// -----------------------------------------------------------------------
	//
	//  ABSTRACT
	//
	//	Note that this is a private constructor
	// -----------------------------------------------------------------------

}

FileEffectiveRightsProbe::~FileEffectiveRightsProbe() {
	// -----------------------------------------------------------------------
	//
	//  ABSTRACT
	//
	//	Do nothing for now
	// -----------------------------------------------------------------------

}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  Public Members  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
AbsProbe* FileEffectiveRightsProbe::Instance() {

	// -----------------------------------------------------------------------
	//
	//  ABSTRACT
	//
	//	Ensure that the FileEffectiveRightsProbe is a singleton.
	// -----------------------------------------------------------------------

	// Use lazy initialization
	if(instance == NULL) 
		instance = new FileEffectiveRightsProbe();

	return instance;	
}

ItemVector* FileEffectiveRightsProbe::CollectItems(Object* object) {
	// -----------------------------------------------------------------------
	//
	//  ABSTRACT
	//
	//	Get all the files on the system that match the pattern reusing the FileFinder.
	//	This probe operates a bit differently than the others. After locating matching
	//	files as the other file related probes do the trustee name is handled as follows:
	//
	//	if not using Variables 
	//	 - if operation == equals
	//		- call GetEffectiveRights method
	//
	//	 - operation == not equal || operation == pattern match
	//		- Get all trustee names on the system. Reuse the logic in SidProbe::GetAllTrusteeNames() 
	//		- Get the set of matching trustee names
	//
	//	- if using variables
	//		- Get all trustee names on the system. Reuse the logic in SidProbe::GetAllTrusteeNames() 
	//		- Get the set of matching trustee names
	//		- call GetEffectiveRights method
	//
	//
	//
	//	TODO:
	//	why are rights always the same?
	//	get sid and domain for trustee name
	//	error handling
	//	memory
	//	should sid code be common? yes...
	//
	// -----------------------------------------------------------------------

	ItemVector *collectedItems = new ItemVector();

	// get the path and file name
	ObjectEntity* path = object->GetElementByName("path");
	ObjectEntity* fileName = object->GetElementByName("filename");
	ObjectEntity* trusteeName = object->GetElementByName("trustee_name");

	// check trustee datatypes - only allow string
	if(trusteeName->GetDatatype() != OvalEnum::STRING_TYPE) {
		throw ProbeException("Error: invalid data type specified on trustee_name. Found: " + OvalEnum::DatatypeToString(trusteeName->GetDatatype()));
	}

	// check trustee operation - only allow  equals, not equals and pattern match
	if(trusteeName->GetOperation() != OvalEnum::EQUALS_OPERATION 
		&& trusteeName->GetOperation() != OvalEnum::PATTERN_MATCH_OPERATION 
		&& trusteeName->GetOperation() != OvalEnum::NOT_EQUAL_OPERATION) {
		throw ProbeException("Error: invalid operation specified on trustee_name. Found: " + OvalEnum::OperationToString(trusteeName->GetOperation()));
	}

	FileFinder fileFinder;
	StringPairVector* filePaths = fileFinder.SearchFiles(path, fileName, object->GetBehaviors());

	// Loop through all file paths deleting them as they are completed.
	while(filePaths->size() > 0) {
		StringPair* fp = filePaths->at(filePaths->size()-1);
		filePaths->pop_back();

		try {
			StringVector* trusteeNames = GetTrusteeNames(fp->first, fp->second, trusteeName);
			StringVector::iterator iterator;
			for(iterator = trusteeNames->begin(); iterator != trusteeNames->end(); iterator++) {
				try {
					Item* item = this->GetEffectiveRights(fp->first, fp->second, (*iterator));
					if(item != NULL) {
						collectedItems->push_back(item);
					}
				} catch (Exception ex) {
					cout << ex.GetErrorMessage() << endl;
				}
			}
			delete trusteeNames;			
		} catch (ProbeException ex) {
			Log::Debug(ex.GetErrorMessage());
		} catch (...) {
			Log::Debug("Unknown error.");
		}
		delete fp;
	}

	return collectedItems;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  Private Members  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//
Item* FileEffectiveRightsProbe::CreateItem() {
	// -----------------------------------------------------------------------
	//
	//  ABSTRACT
	//
	//  Return a new Item created for storing file information
	//
	// -----------------------------------------------------------------------

	Item* item = new Item(0, 
						"http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#windows", 
						"win-sc", 
						"http://oval.mitre.org/XMLSchema/oval-system-characteristics-5#windows windows-system-characteristics-schema.xsd", 
						OvalEnum::ERROR_STATUS, 
						"fileeffectiverights_item");

	return item;
}

StringVector* FileEffectiveRightsProbe::GetTrusteeNames(string path, string fileName, ObjectEntity* trusteeName) {
	// -----------------------------------------------------------------------
	//  ABSTRACT
	//
	//	Return the set of trustee names that are specified by the 
	//  ObjectEntity* trusteeName parameter.
	// -----------------------------------------------------------------------

	StringVector* trusteeNames = new StringVector();
	
	// load the set of all Trustee names
	StringVector* allTrusteeNames = WindowsCommon::GetAllTrusteeNames();

	// does this trusteeName use variables?
	if(trusteeName->GetVarRef() == NULL) {
		
		// proceed based on operation
		if(trusteeName->GetOperation() == OvalEnum::EQUALS_OPERATION) {
			
			// check that the trusee name exists
			if(this->TrusteeNameExists(trusteeName->GetValue(), allTrusteeNames)) {
				trusteeNames->push_back(trusteeName->GetValue());
			}

		} else if(trusteeName->GetOperation() == OvalEnum::NOT_EQUAL_OPERATION) {
			this->GetMatchingTrusteeNames(trusteeName->GetValue(), allTrusteeNames, trusteeNames, false);
		} else if(trusteeName->GetOperation() == OvalEnum::PATTERN_MATCH_OPERATION) {
			this->GetMatchingTrusteeNames(trusteeName->GetValue(), allTrusteeNames, trusteeNames, true);
		}		

	} else {
	
		// loop through all trustee names on the system
		// only keep those that match operation and value and var check
		StringVector::iterator it;
		ItemEntity* tmp = this->CreateItemEntity(trusteeName);
		for(it = allTrusteeNames->begin(); it != allTrusteeNames->end(); it++) {
			tmp->SetValue((*it));
			if(trusteeName->Analyze(tmp) == OvalEnum::TRUE_RESULT) {
				trusteeNames->push_back((*it));
			}
		}
	}

	// using common code to get these do not delete they are cached.
	//delete allTrusteeNames;

	return trusteeNames;
}

void FileEffectiveRightsProbe::GetMatchingTrusteeNames(string trusteeNamePattern, StringVector* allTrusteeNames, StringVector* trusteeNames, bool isRegex) {
	// -----------------------------------------------------------------------
	//
	//  ABSTRACT
	//
	//	Select all Trustees for the specified file that match the specified
	//	trusteeNamePattern. matching is done via regex or != depending on the
	//	isRegex boolean flag. All matches are added to the trusteeNames vector.
	// -----------------------------------------------------------------------

	StringVector::iterator iterator;
	for(iterator = allTrusteeNames->begin(); iterator != allTrusteeNames->end(); iterator++) {
		if(this->IsMatch(trusteeNamePattern, (*iterator), isRegex)) {
			trusteeNames->push_back((*iterator));
		}
	}
}

bool FileEffectiveRightsProbe::TrusteeNameExists(string trusteeName, StringVector* trusteeNames) {
	// -----------------------------------------------------------------------
	//
	//  ABSTRACT
	//
	//	Return a true if the specified trustee name exists in the set of all names.  
	// -----------------------------------------------------------------------

	bool exists = false;

	StringVector::iterator iterator;
	for(iterator = trusteeNames->begin(); iterator != trusteeNames->end(); iterator++) {
		if(trusteeName.compare((*iterator)) == 0) {
			exists = true;
		}
	}			

	return exists;
}

// this attempt uses GetFileSecurity a little differently
//Item* FileEffectiveRightsProbe::GetEffectiveRights(string path, string fileName, string trusteeName) {
//	// -----------------------------------------------------------------------
//	//
//	//  ABSTRACT
//	//
//	//	Return a populated item for the specified trustess on the specified file.
//	//
//	//	- Use provided trustee name and call LsaLookupNames to get the sid
//	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secmgmt/security/lsalookupnames.asp
//	//	- Then call GetNamedSecurityInfo to get a DACL Security Descriptor for the file
//	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/getnamedsecurityinfo.asp
//	//	- Then call GetEffectiveRightsFromAcl with the dacl and the sid found in the earlier calls
//	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/geteffectiverightsfromacl.asp
//	// -----------------------------------------------------------------------
//	
//	Item* item = NULL;
//
//	// build the path
//	string filePath = path;
//	if(fileName.compare("") != 0) {
//		// Verify that the path that was passed into this function ends with a slash.  If
//		// it doesn't, then add one.
//		if (path[path.length()-1] != Common::fileSeperator)
//			filePath.append(1, Common::fileSeperator);
//
//		filePath.append(fileName);
//	}
//
//	BOOL bSuccess = TRUE;
//	BYTE* pSecDescriptorBuf;
//	DWORD dwSizeNeeded = 0;
//
//	// Find out size of needed buffer for security descriptor with DACL
//	// DACL = Discretionary Access Control List
//	bSuccess = GetFileSecurity(const_cast<char*>(filePath.c_str()),
//								DACL_SECURITY_INFORMATION,
//								NULL,
//								0,
//								&dwSizeNeeded);
//
//	if (0 == dwSizeNeeded) {
//		// throw error
//		cout << "error returne needed size of 0 " << endl;
//	}
//	pSecDescriptorBuf = new BYTE[dwSizeNeeded];
//
//	// Retrieve security descriptor with DACL information
//	bSuccess = GetFileSecurity(const_cast<char*>(filePath.c_str()),
//								DACL_SECURITY_INFORMATION,
//								pSecDescriptorBuf,
//								dwSizeNeeded,
//								&dwSizeNeeded);
//
//	// Check if we successfully retrieved security descriptor with DACL information
//	if (!bSuccess) {
//		DWORD dwError = GetLastError();
//		cout << "Failed to get file security information (" << dwError << ")\n";
//		// throw this
//	}
//	
//	// Getting DACL from Security Descriptor
//	PACL pdacl;
//	BOOL bDaclPresent, bDaclDefaulted;
//	bSuccess = GetSecurityDescriptorDacl((SECURITY_DESCRIPTOR*)pSecDescriptorBuf,
//										 &bDaclPresent, &pdacl, &bDaclDefaulted);
//	
//	// Check if we successfully retrieved DACL
//	if (!bSuccess) {
//		DWORD dwError = GetLastError();
//		cout << "Failed to retrieve DACL from security descriptor (" << dwError << ")\n";
//		// throw this
//	}
//	
//	// Check if DACL present in security descriptor
//	if (!bDaclPresent) {
//		cout << "DACL was not found.\n";
//		// throw this
//	}
//
//	// Get the sid for the trustee name
//	PSID pSid = this->GetSIDForTrusteeName(trusteeName);
//
//	// check the sid
//	if(!IsValidSid(pSid)) {
//		cout << "error invalid sid for truste name: " + trusteeName << endl;
//	}
//
//	// build the trustee structure
//	TRUSTEE trustee = {0};
//	BuildTrusteeWithSid(&trustee, pSid);
//
//	// get the rights
//	PACCESS_MASK accessRights = NULL;
//	accessRights = reinterpret_cast<PACCESS_MASK>(::LocalAlloc(LPTR, sizeof(PACCESS_MASK) + sizeof(ACCESS_MASK)));
//	if(accessRights == NULL) {
//		ProbeException("Out of memory");
//	}
//
//	DWORD res = GetEffectiveRightsFromAcl(pdacl,
//									&trustee,
//									accessRights);
//	if (res != ERROR_SUCCESS) {
//		free(pSecDescriptorBuf);
//		throw ProbeException("Error unable to get effictive rights for trustee: " + trusteeName + " from dacl for file: " + filePath); 
//	} 
//		
//	// Convert access mask to binary.
//	// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/access_mask_format.asp
//	int j;
//	char mask[33];
//
//	for (j = 0; j < 32; j++) {
//		if ((*accessRights) & (1 << j))
//			mask[31 - j] = '1';
//		else
//			mask[31 - j] = '0';
//	}
//	mask[32] = 0;
//
//	item = this->CreateItem();
//	item->SetStatus(OvalEnum::EXISTS_STATUS);
//	item->AppendElement(new ItemEntity("path", path, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("filename", fileName, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("trustee_name", trusteeName, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//
//	
//	// split the string at the '\'
//	//int pos = trusteeName.find("\\", 0);
//	//string trusteeDomain = "";
//	//if(pos != -1) {
//	//	if(pos != 0) {
//	//		trusteeDomain = trusteeName.substr(0, pos);
//	//	}
//	//}
//	item->AppendElement(new ItemEntity("trustee_domain", "", OvalEnum::STRING_TYPE, true, OvalEnum::NOT_COLLECTED_STATUS));
//	// need to get trustee_sid
//	item->AppendElement(new ItemEntity("trustee_sid", "", OvalEnum::STRING_TYPE, true, OvalEnum::NOT_COLLECTED_STATUS));
//
//	// read values in the access_mask
//	// http://windowssdk.msdn.microsoft.com/en-us/library/ms717896.aspx
//	item->AppendElement(new ItemEntity("standard_delete", Common::ToString(mask[16]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_read_control", Common::ToString(mask[17]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_write_dac", Common::ToString(mask[18]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_write_owner", Common::ToString(mask[19]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_synchronize", Common::ToString(mask[20]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("access_system_security", Common::ToString(mask[24]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_read", Common::ToString(mask[31]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_write", Common::ToString(mask[30]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_execute", Common::ToString(mask[29]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_all", Common::ToString(mask[28]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_data", Common::ToString(mask[0]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_data", Common::ToString(mask[1]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_append_data", Common::ToString(mask[2]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_ea", Common::ToString(mask[3]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_ea", Common::ToString(mask[4]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_execute", Common::ToString(mask[5]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_delete_child", Common::ToString(mask[6]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_attributes", Common::ToString(mask[7]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_attributes", Common::ToString(mask[8]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//
//	free(pSecDescriptorBuf);
//
//	return item;
//}

// this attempt uses GetFileSecurity
//Item* FileEffectiveRightsProbe::GetEffectiveRights(string path, string fileName, string trusteeName) {
//	// -----------------------------------------------------------------------
//	//
//	//  ABSTRACT
//	//
//	//	Return a populated item for the specified trustess on the specified file.
//	//
//	//	- Use provided trustee name and call LsaLookupNames to get the sid
//	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secmgmt/security/lsalookupnames.asp
//	//	- Then call GetNamedSecurityInfo to get a DACL Security Descriptor for the file
//	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/getnamedsecurityinfo.asp
//	//	- Then call GetEffectiveRightsFromAcl with the dacl and the sid found in the earlier calls
//	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/geteffectiverightsfromacl.asp
//	// -----------------------------------------------------------------------
//	
//	Item* item = NULL;
//
//	// build the path
//	string filePath = path;
//	if(fileName.compare("") != 0) {
//		// Verify that the path that was passed into this function ends with a slash.  If
//		// it doesn't, then add one.
//		if (path[path.length()-1] != Common::fileSeperator)
//			filePath.append(1, Common::fileSeperator);
//
//		filePath.append(fileName);
//	}
//
//	unsigned long sdRequiredSize = 0;
//	GetFileSecurity(const_cast<char*>(filePath.c_str()),	// file name
//					DACL_SECURITY_INFORMATION,				// type of info to get
//					0,										// the security descriptor
//					0,										// size of sd
//					&sdRequiredSize);						// required size of the sd
//
//	UINT uRet = GetLastError();
//	LPBYTE lpSecurityBuffer = NULL;
//
//	if(uRet==ERROR_INSUFFICIENT_BUFFER && sdRequiredSize) {
//		// Clear the ERROR_INSUFFICIENT_BUFFER
//		uRet = S_OK; 
//
//		// Allocate the buffer for the Security Descriptor, check for out of memory
//		lpSecurityBuffer = (LPBYTE) malloc(sdRequiredSize * sizeof(BYTE));
//		if(!lpSecurityBuffer) {
//			// thorw exception here not enought memory
//			cout << "not enough memory." << endl;
//		}
//		
//	} else {
//		// some sort of error otehr than i need to allocate memory...
//		cout << WindowsCommon::GetErrorMessage(GetLastError()) << endl;
//	}
//
//	// get the sd
//	BOOL gfsRes = GetFileSecurity(const_cast<char*>(filePath.c_str()),	// file name
//							DACL_SECURITY_INFORMATION,				// type of info to get
//							lpSecurityBuffer,						// the security descriptor
//							sdRequiredSize,							// size of sd
//							&sdRequiredSize);						// required size of the sd
//
//	if(gfsRes == 0) {
//		cout << " error in call to GetFileSecurity" << WindowsCommon::GetErrorMessage(GetLastError()) << endl;
//	}
//
//	DWORD dwErr = 0;
//	BOOL AclPresent = FALSE, AclDefaulted = FALSE;
//	PACL pdacl = NULL;
//	// Get the Acl from the SD given
//	dwErr = GetSecurityDescriptorDacl(lpSecurityBuffer, &AclPresent, &pdacl, &AclDefaulted);
//	if(pdacl == NULL || dwErr == FALSE) {
//		cout << "error while getting dacl from security descriptor  " + WindowsCommon::GetErrorMessage(GetLastError()) << endl;
//	}
//
//	// Check to see if a valid security descriptor were returned.  
//    if ((IsValidSecurityDescriptor(lpSecurityBuffer) == 0) || (IsValidAcl(pdacl) == 0)  /*|| (IsValidSid(pOwnersid) == 0) || (IsValidSid(pPrimaryGroupdsid) == 0)*/) {
//		free(lpSecurityBuffer);
//		//LocalFree(pSD);
//		throw ProbeException("Error invalid data returned from call to GetNamedSecurityInfo(). Cannot get effective rights for file: " + filePath);
//	}
//
//	// Get the sid for the trustee name
//	PSID pSid = this->GetSIDForTrusteeName(trusteeName);
//
//	// check the sid
//	if(!IsValidSid(pSid)) {
//		cout << "error invalid sid for truste name: " + trusteeName << endl;
//	}
//
//	// build the trustee structure
//	TRUSTEE trustee = {0};
//	BuildTrusteeWithSid(&trustee, pSid);
//
//	// get the rights
//	PACCESS_MASK accessRights = NULL;
//	accessRights = reinterpret_cast<PACCESS_MASK>(::LocalAlloc(LPTR, sizeof(PACCESS_MASK) + sizeof(ACCESS_MASK)));
//	if(accessRights == NULL) {
//		ProbeException("Out of memory");
//	}
//
//	DWORD res = GetEffectiveRightsFromAcl(pdacl,
//									&trustee,
//									accessRights);
//	if (res != ERROR_SUCCESS) {
//		free(lpSecurityBuffer);
//		throw ProbeException("Error unable to get effictive rights for trustee: " + trusteeName + " from dacl for file: " + filePath); 
//	} 
//		
//	// Convert access mask to binary.
//	// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/access_mask_format.asp
//	int j;
//	char mask[33];
//
//	for (j = 0; j < 32; j++) {
//		if ((*accessRights) & (1 << j))
//			mask[31 - j] = '1';
//		else
//			mask[31 - j] = '0';
//	}
//	mask[32] = 0;
//
//	item = this->CreateItem();
//	item->SetStatus(OvalEnum::EXISTS_STATUS);
//	item->AppendElement(new ItemEntity("path", path, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("filename", fileName, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("trustee_name", trusteeName, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//
//	
//	// split the string at the '\'
//	//int pos = trusteeName.find("\\", 0);
//	//string trusteeDomain = "";
//	//if(pos != -1) {
//	//	if(pos != 0) {
//	//		trusteeDomain = trusteeName.substr(0, pos);
//	//	}
//	//}
//	item->AppendElement(new ItemEntity("trustee_domain", "", OvalEnum::STRING_TYPE, true, OvalEnum::NOT_COLLECTED_STATUS));
//	// need to get trustee_sid
//	item->AppendElement(new ItemEntity("trustee_sid", "", OvalEnum::STRING_TYPE, true, OvalEnum::NOT_COLLECTED_STATUS));
//
//	// read values in the access_mask
//	// http://windowssdk.msdn.microsoft.com/en-us/library/ms717896.aspx
//	item->AppendElement(new ItemEntity("standard_delete", Common::ToString(mask[16]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_read_control", Common::ToString(mask[17]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_write_dac", Common::ToString(mask[18]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_write_owner", Common::ToString(mask[19]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_synchronize", Common::ToString(mask[20]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("access_system_security", Common::ToString(mask[24]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_read", Common::ToString(mask[31]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_write", Common::ToString(mask[30]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_execute", Common::ToString(mask[29]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_all", Common::ToString(mask[28]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_data", Common::ToString(mask[0]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_data", Common::ToString(mask[1]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_append_data", Common::ToString(mask[2]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_ea", Common::ToString(mask[3]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_ea", Common::ToString(mask[4]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_execute", Common::ToString(mask[5]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_delete_child", Common::ToString(mask[6]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_attributes", Common::ToString(mask[7]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_attributes", Common::ToString(mask[8]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//
//	free(lpSecurityBuffer);
//
//	return item;
//}

// this attempt uses GetNamedSecurityInfo and only gives me generic rights
//
Item* FileEffectiveRightsProbe::GetEffectiveRights(string path, string fileName, string trusteeName) {
	// -----------------------------------------------------------------------
	//
	//  ABSTRACT
	//
	//	Return a populated item for the specified trustess on the specified file.
	//
	//	- Use provided trustee name and call LsaLookupNames to get the sid
	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secmgmt/security/lsalookupnames.asp
	//	- Then call GetNamedSecurityInfo to get a DACL Security Descriptor for the file
	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/getnamedsecurityinfo.asp
	//	- Then call GetEffectiveRightsFromAcl with the dacl and the sid found in the earlier calls
	//	  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/geteffectiverightsfromacl.asp
	// -----------------------------------------------------------------------
	
	Item* item = NULL;

	// build the path
	string filePath = path;
	if(fileName.compare("") != 0) {
		// Verify that the path that was passed into this function ends with a slash.  If
		// it doesn't, then add one.
		if (path[path.length()-1] != Common::fileSeperator)
			filePath.append(1, Common::fileSeperator);

		filePath.append(fileName);
	}

	DWORD res;
	PACL pdacl;
	PSECURITY_DESCRIPTOR pSD;

	res = GetNamedSecurityInfo(const_cast<char*>(filePath.c_str()),	// object name
							   SE_FILE_OBJECT,						// object type
							   DACL_SECURITY_INFORMATION, 			// information type
							   NULL,								// owner SID
							   NULL,								// primary group SID
							   &pdacl,								// DACL
							   NULL,								// SACL
							   &pSD);								// Security Descriptor

	if (res != ERROR_SUCCESS) {
		if (res == ERROR_FILE_NOT_FOUND) {
			// should never get here. before calling this function the file should already have 
			// been checked for existance.
			throw ProbeException("Error unable locate " + filePath + " while getting trustte names."); 
		} else {
			throw ProbeException("Error unable to retrieve a copy of the security descriptor for " + filePath + " while getting trustee names."); 
		}
	} 

	// Check to see if a valid security descriptor were returned.  
    if ((IsValidSecurityDescriptor(pSD) == 0) || (IsValidAcl(pdacl) == 0)) {
		LocalFree(pSD);
		throw ProbeException("Error invalid data returned from call to GetNamedSecurityInfo(). Cannot get effective rights for file: " + filePath);
	}

	// Get the sid for the trustee name
	PSID pSid = WindowsCommon::GetSIDForTrusteeName(trusteeName);

	// check the sid
	if(!IsValidSid(pSid)) {
		cout << "error invalid sid for trustee: " +  trusteeName << endl;
	}

	// build the trustee structure
	TRUSTEE trustee = {0};
	BuildTrusteeWithSid(&trustee, pSid);

	// get the rights
	PACCESS_MASK accessRights = NULL;
	accessRights = reinterpret_cast<PACCESS_MASK>(::LocalAlloc(LPTR, sizeof(PACCESS_MASK) + sizeof(ACCESS_MASK)));
	if(accessRights == NULL) {
		ProbeException("Out of memory");
	}

	res = GetEffectiveRightsFromAcl(pdacl,
									&trustee,
									accessRights);
	if (res != ERROR_SUCCESS) {
		LocalFree(pSD);
		throw ProbeException("Error unable to get effictive rights for trustee: " + trusteeName + " from dacl for file: " + filePath); 
	} 
		
	// Convert access mask to binary.
	// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/access_mask_format.asp
	int j;
	char mask[33];

	for (j = 0; j < 32; j++) {
		if ((*accessRights) & (1 << j))
			mask[31 - j] = '1';
		else
			mask[31 - j] = '0';
	}
	mask[32] = 0;

	item = this->CreateItem();
	item->SetStatus(OvalEnum::EXISTS_STATUS);
	item->AppendElement(new ItemEntity("path", path, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("filename", fileName, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("trustee_name", trusteeName, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));

	string domainStr = "";
	string sidStr = "";
	bool isGroup = WindowsCommon::LookUpTrusteeName(&trusteeName, &sidStr, &domainStr);
	item->AppendElement(new ItemEntity("trustee_domain", domainStr, OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("trustee_sid", sidStr, OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));

	// read values in the access_mask
	// http://windowssdk.msdn.microsoft.com/en-us/library/ms717896.aspx
	item->AppendElement(new ItemEntity("standard_delete", Common::ToString(mask[16]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("standard_read_control", Common::ToString(mask[17]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("standard_write_dac", Common::ToString(mask[18]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("standard_write_owner", Common::ToString(mask[19]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("standard_synchronize", Common::ToString(mask[20]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("access_system_security", Common::ToString(mask[24]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("generic_read", Common::ToString(mask[31]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("generic_write", Common::ToString(mask[30]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("generic_execute", Common::ToString(mask[29]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("generic_all", Common::ToString(mask[28]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_read_data", Common::ToString(mask[0]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_write_data", Common::ToString(mask[1]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_append_data", Common::ToString(mask[2]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_read_ea", Common::ToString(mask[3]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_write_ea", Common::ToString(mask[4]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_execute", Common::ToString(mask[5]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_delete_child", Common::ToString(mask[6]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_read_attributes", Common::ToString(mask[7]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
	item->AppendElement(new ItemEntity("file_write_attributes", Common::ToString(mask[8]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));

	LocalFree(pSD);

	return item;
}

//PSID FileEffectiveRightsProbe::GetSIDForTrusteeName(string trusteeName) {
//	// -----------------------------------------------------------------------
//	//
//	//  ABSTRACT
//	//
//	//	Return the SID for the specified trustee name.
//	//	TrusteeName should be a fully qualified account name. For more info 
//	//	see:
//	//	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/lookupaccountname.asp
//	//
//	// -----------------------------------------------------------------------
//	
//	DWORD sidSize = 128;
//	DWORD domainSize = 128;
//	SID_NAME_USE sidUse;
//	BOOL retVal = FALSE;
//	PSID psid = NULL;
//	LPTSTR domain = NULL;
//
//	try {
//		do {
//			// Initial memory allocations for the SID and DOMAIN.
//			psid = (PSID)realloc(psid, sidSize);
//			if (psid == NULL) {
//				retVal = FALSE;
//				break;
//			}
//
//			domain = (LPTSTR)realloc(domain, domainSize);
//			if (domain == NULL) {
//				retVal = FALSE;
//				break;
//			}
//
//			// Call LookupAccountName to get the SID.
//			retVal = LookupAccountName(NULL,								// system name NULL == localhost
//									const_cast<char*>(trusteeName.c_str()),	// account name
//									psid,									// security identifier
//									&sidSize,								// size of security identifier
//									domain,									// domain name
//									&domainSize,							// size of domain name
//									&sidUse);								// SID-type indicator
//
//		} while (GetLastError() == ERROR_INSUFFICIENT_BUFFER);
//
//	} catch(...) {
//		
//		Log::Debug("Error looking up sid for account. " + WindowsCommon::GetErrorMessage(GetLastError()));
//	}
//
//	return psid;
//}

//
//bool FileEffectiveRightsProbe::EnablePrivilege(string privilegeIn) {
//	//------------------------------------------------------------------------------------//
//	//  ABSTRACT
//	//
//	//  This gives us a privilege.
//	//
//	//	Seems like i am not really able to elevate the privilages of the current process.
//	//	the error message i get is of course misleading
//	//
//	//------------------------------------------------------------------------------------//
//
//	TOKEN_PRIVILEGES tp;
//	HANDLE hProcess = NULL;
//	HANDLE hAccessToken = NULL;
//
//	hProcess = GetCurrentProcess();
//
//	if(!OpenProcessToken(hProcess,									// handle to the process
//						(TOKEN_QUERY | TOKEN_ADJUST_PRIVILEGES),	// requested access types 
//						&hAccessToken) == FALSE)					// new access token 
//	{
//		string errmsg = WindowsCommon::GetErrorMessage(GetLastError());
//
//		return false;
//	}
//
//	tp.PrivilegeCount = 1;
//    tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
//    
//	if (LookupPrivilegeValue(NULL, privilegeIn.c_str(), &tp.Privileges[0].Luid) == 0) {
//		return false;
//	}
//
//	if (AdjustTokenPrivileges(hAccessToken, FALSE, &tp, NULL, NULL, NULL) == 0) {
//		return false;
//	}
//	 
//	if(GetLastError() == ERROR_NOT_ALL_ASSIGNED) {
//		// The token for the current process does not have the privilege specified. The
//		// AdjustTokenPrivileges() function may succeed with this error value even if no
//		// privileges were adjusted.  The privilege parameter can specify privileges that
//		// the token does not have, without causing the function to fail. In this case, 
//		// the function adjusts the privileges that the token does have and ignores the 
//		// other privileges so that the function succeeds.
//
//		CloseHandle(hAccessToken);
//		return false;
//	} else {
//		CloseHandle(hAccessToken);
//		return true;
//	}
//}
//
//bool FileEffectiveRightsProbe::DisablePrivilege(string privilegeIn)
//{
//	//------------------------------------------------------------------------------------//
//	//  ABSTRACT
//	//
//	//  This function disables a privilege associated with the current process
//	//  token. 
//	//
//	//------------------------------------------------------------------------------------//
//
//	TOKEN_PRIVILEGES tp;
//	HANDLE hProcess = NULL;
//	HANDLE hAccessToken = NULL;
//
//	hProcess = GetCurrentProcess();
//
//	if(!OpenProcessToken(hProcess,									// handle to the process
//						(TOKEN_QUERY | TOKEN_ADJUST_PRIVILEGES),	// requested access types 
//						&hAccessToken) == FALSE)					// new access token 
//	{
//		return false;
//	}
//
//	tp.PrivilegeCount = 1;
//    tp.Privileges[0].Attributes = SE_PRIVILEGE_REMOVED;
//    
//	if (LookupPrivilegeValue(NULL, privilegeIn.c_str(), &tp.Privileges[0].Luid) == 0) {
//		return false;
//	}
//
//	if (AdjustTokenPrivileges(hAccessToken, FALSE, &tp, NULL, NULL, NULL) == 0) {
//		return false;
//	}
//	 
//	if(GetLastError() == ERROR_NOT_ALL_ASSIGNED) {
//		// The token for the current process does not have the privilege specified. The
//		// AdjustTokenPrivileges() function may succeed with this error value even if no
//		// privileges were adjusted.  The privilege parameter can specify privileges that
//		// the token does not have, without causing the function to fail. In this case, 
//		// the function adjusts the privileges that the token does have and ignores the 
//		// other privileges so that the function succeeds.
//
//		CloseHandle(hAccessToken);
//		return false;
//	} else {
//		CloseHandle(hAccessToken);
//		return true;
//	}
//}


//	old version now using code from sidprobe to find all trustees.
//
//StringVector* FileEffectiveRightsProbe::GetAllTrusteeNames(string path, string fileName) {
//	// -----------------------------------------------------------------------
//	//
//	//  ABSTRACT
//	//
//	//	Return a StringVector* of all Trustees for the specified file. 
//	//	Looks only at owner, primary group, and dacl.
//	// -----------------------------------------------------------------------
//
//	StringVector* trusteeNames = new StringVector();
//	UniqueStringVector* usvTrusteeNames = new UniqueStringVector(trusteeNames);
//
//	// Verify that the path that was passed into this function ends with a slash.  If
//	// it doesn't, then add one.
//	if (path[path.length()-1] != Common::fileSeperator)
//		path.append(1, Common::fileSeperator);
//
//	string filePath = path + fileName;
//
//	DWORD res;
//	PACL pdacl;
//	PSID pPrimaryGroupdsid;
//	PSID pOwnersid;
//	PSECURITY_DESCRIPTOR pSD;
//
//	res = GetNamedSecurityInfo(const_cast<char*>(filePath.c_str()),	// object name
//							   SE_FILE_OBJECT,						// object type
//							   DACL_SECURITY_INFORMATION |			// information type
//							   GROUP_SECURITY_INFORMATION |
//							   OWNER_SECURITY_INFORMATION, 
//							   &pOwnersid,							// owner SID
//							   &pPrimaryGroupdsid,					// primary group SID
//							   &pdacl,								// DACL
//							   NULL,								// SACL
//							   &pSD);								// Security Descriptor
//
//	if (res != ERROR_SUCCESS) {
//		if (res == ERROR_FILE_NOT_FOUND) {
//			// should never get here. before calling this function the file should already have 
//			// been checked for existance.
//			throw ProbeException("Error unable locate " + filePath + " while getting trustte names."); 
//		} else {
//			throw ProbeException("Error unable to retrieve a copy of the security descriptor for " + filePath + " while getting trustee names."); 
//		}
//	} 
//
//	// Check to see if a valid security descriptor were returned.  
//    if ((IsValidSecurityDescriptor(pSD) == 0) || (IsValidAcl(pdacl) == 0)  || (IsValidSid(pOwnersid) == 0) || (IsValidSid(pPrimaryGroupdsid) == 0)) {
//		LocalFree(pSD);
//		throw ProbeException("Error invalid data returned from call to GetNamedSecurityInfo(). Cannot get effective rights for file: " + filePath);
//	}
//
//	// handle owner and primary group sids
//	string ownerTrusteeName = "";
//	string ownerTrusteeDomain = "";
//	string ownerTrusteeSid = "";
//	this->GetTrusteeInfoForSid(pOwnersid, &ownerTrusteeName, &ownerTrusteeDomain, &ownerTrusteeSid);
//	usvTrusteeNames->Append(ownerTrusteeName);
//
//	string groupTrusteeName = "";
//	string groupTrusteeDomain = "";
//	string groupTrusteeSid = "";
//	this->GetTrusteeInfoForSid(pPrimaryGroupdsid, &groupTrusteeName, &groupTrusteeDomain, &groupTrusteeSid);
//	usvTrusteeNames->Append(groupTrusteeName);
//
//	// process dacl
//	ACL_SIZE_INFORMATION acl_size_info;
//	DWORD acl_size = sizeof(ACL_SIZE_INFORMATION);
//
//	if (GetAclInformation(pdacl,						// access-control list
//						  (LPVOID)&acl_size_info,		// ACL information
//						  acl_size,						// size of ACL information
//						  AclSizeInformation) != 0)		// info class
//	{
//
//		// Loop through each ACE found in the ACL.
//		DWORD ace_number;
//		for (ace_number = 0; ace_number < acl_size_info.AceCount; ace_number++) {
//			// Get the ACE.
//			ACCESS_ALLOWED_ACE* ace;
//            if (GetAce(pdacl, ace_number, (LPVOID *) & ace) == 0) {
//				// how should this error be handled?
//				//throw ProbeException("Unable to get next ACE for: " + filePath);
//				continue;
//            }
//
//			// Get the SID associated with this ACE.
//			PSID psid = (PSID) & (ace->SidStart);
//
//			string trusteeName = "";
//			string trusteeDomain = "";
//			string trusteeSid = "";
//			this->GetTrusteeInfoForSid(psid, &trusteeName, &trusteeDomain, &trusteeSid);				
//			usvTrusteeNames->Append(trusteeName);
//		}
//
//		LocalFree(pSD);
//	} else {
//		LocalFree(pSD);
//		throw ProbeException("Unable to retrieve access-control list for: " + filePath);
//	}
//
//	return trusteeNames;
//}

// no longer used
//
//void FileEffectiveRightsProbe::GetTrusteeInfoForSid(PSID psid, string* trusteeName, string* trusteeDomain, string* trusteeSid) {
//	// -----------------------------------------------------------------------
//	//
//	//  ABSTRACT
//	//
//	//	For the specified SID get all trustee info.
//	//	Log any error getting name and domain and then return an empty string	
//	// -----------------------------------------------------------------------
//
//	// Check to make sure the SID is valid.
//    if (!(IsValidSid(psid))) {
//		throw ProbeException("Error Invalid SID.");
//    }
//
//	// get the trusee_sid
//	LPTSTR sid_string;
//	Common::GetTextualSid(psid, &sid_string);
//	(*trusteeSid) = sid_string;
//	free(sid_string);
//
//	// Call LookupAccountSid with owner_name_size = 0 and domain_name_size =0.
//	// This will cause LookupAccountSid to fail and return with the required
//	// buffer sizes.
//	SID_NAME_USE sid_type;
//	LPTSTR trustee_name = NULL;
//	LPTSTR domain_name = NULL;
//	DWORD trustee_name_size = 0;
//	DWORD domain_name_size = 0;
//
//	LookupAccountSid(NULL,						// name of local or remote computer
//						psid,					// security identifier
//						trustee_name,			// account name buffer
//						&trustee_name_size,		// size of account name buffer
//						domain_name,			// domain name
//						&domain_name_size,		// size of domain name buffer
//						&sid_type);				// SID type
//
//    trustee_name_size++;
//    trustee_name = (LPTSTR)realloc(trustee_name, trustee_name_size * sizeof(TCHAR));
//    if (trustee_name == NULL) {
//		throw ProbeException("Could not allocate space. Cannot get trustee_name for sid.");
//    }
//
//    domain_name_size++;
//    domain_name = (LPTSTR)realloc(domain_name, domain_name_size * sizeof(TCHAR));
//	if (domain_name == NULL) {
//		throw ProbeException("Could not allocate space. Cannot get domain_name for.");
//    }
//	
//	// Call LookupAccountSid again to retrieve the name of the account and the
//	// name of the first domain on which this SID is found.
//    if (LookupAccountSid(NULL,						// name of local or remote computer
//							psid,					// security identifier
//							trustee_name,			// account name buffer
//							&trustee_name_size,		// size of account name buffer
//							domain_name,			// domain name
//							&domain_name_size,		// size of domain name buffer
//							&sid_type) == 0)		// SID type
//	{
//		string errMsg = WindowsCommon::GetErrorMessage(GetLastError());
//		Log::Debug("Unable to get the trustee name and domain of the account for this SID. " + errMsg);
//
//		// TODO: how should these error messages be handled?
//		(*trusteeDomain) = "";
//		(*trusteeName) = "";
//
//	} else {
//		(*trusteeDomain) = domain_name;
//		(*trusteeName).append(domain_name);
//		(*trusteeName).append("\\");
//		(*trusteeName).append(trustee_name);
//	}
//
//	free(domain_name);
//	free(trustee_name);
//
//	// now expand any groups to get all user sids too
//	if(sid_type == SidTypeUser) {
//		cout << "user" << endl;
//	} else if(sid_type == SidTypeGroup) {
//		cout << "group - expand" << endl;
//	} else if(sid_type == SidTypeDomain) {
//		cout << "domain" << endl;
//	} else if(sid_type == SidTypeAlias) {
//		cout << "alias" << endl;
//	} else if(sid_type == SidTypeWellKnownGroup) {
//		cout << "well known group - expand" << endl;
//	} else if(sid_type == SidTypeDeletedAccount) {
//		cout << "deleted account" << endl;
//	} else if(sid_type == SidTypeInvalid) {
//		cout << "invalid" << endl;
//	} else if(sid_type == SidTypeUnknown) {
//		cout << "unknwon" << endl;
//	} else if(sid_type == SidTypeComputer) {
//		cout << "computer" << endl;
//	}  
//}


// not used
//
//PSID FileEffectiveRightsProbe::GetSIDForTrusteeName(string trusteeName) {
//	// -----------------------------------------------------------------------
//	//
//	//  ABSTRACT
//	//
//	//	Return the SID for the specified trustee name.
//	//	TrusteeName should be a fully qualified account name. For more info 
//	//	see:
//	//	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/secauthz/security/lookupaccountname.asp
//	//
//	// -----------------------------------------------------------------------
//	
//	DWORD sidSize = 128;
//	DWORD domainSize = 128;
//	SID_NAME_USE sidUse;
//	BOOL retVal = FALSE;
//	PSID psid = NULL;
//	LPTSTR domain = NULL;
//
//	try {
//		do {
//			// Initial memory allocations for the SID and DOMAIN.
//			psid = (PSID)realloc(psid, sidSize);
//			if (psid == NULL) {
//				retVal = FALSE;
//				break;
//			}
//
//			domain = (LPTSTR)realloc(domain, domainSize);
//			if (domain == NULL) {
//				retVal = FALSE;
//				break;
//			}
//
//			// Call LookupAccountName to get the SID.
//			retVal = LookupAccountName(NULL,								// system name NULL == localhost
//									const_cast<char*>(trusteeName.c_str()),	// account name
//									psid,									// security identifier
//									&sidSize,								// size of security identifier
//									domain,									// domain name
//									&domainSize,							// size of domain name
//									&sidUse);								// SID-type indicator
//
//		} while (GetLastError() == ERROR_INSUFFICIENT_BUFFER);
//
//	} catch(...) {
//		
//		Log::Debug("Error looking up sid for account. " + WindowsCommon::GetErrorMessage(GetLastError()));
//	}
//
//	return psid;
//}

// not used
//
//Item* FileEffectiveRightsProbe::GetEffectiveRightsForSID(string path, string fileName, PSID psid) {
//	// -----------------------------------------------------------------------
//	//
//	//  ABSTRACT
//	//
//	//	Return a populated item for the specified truess on the specifeid file.
//	// -----------------------------------------------------------------------
//	
//	Item* item = NULL;
//
//	// Verify that the path that was passed into this function ends with a slash.  If
//	// it doesn't, then add one.
//	if (path[path.length()-1] != Common::fileSeperator)
//		path.append(1, Common::fileSeperator);
//
//	string filePath = path + fileName;
//
//	DWORD res;
//	PACL pdacl;
//	PSID pPrimaryGroupdsid;
//	PSID pOwnersid;
//	PSECURITY_DESCRIPTOR pSD;
//
//	res = GetNamedSecurityInfo(const_cast<char*>(filePath.c_str()),	// object name
//							   SE_FILE_OBJECT,						// object type
//							   DACL_SECURITY_INFORMATION |			// information type
//							   GROUP_SECURITY_INFORMATION |
//							   OWNER_SECURITY_INFORMATION, 
//							   &pOwnersid,							// owner SID
//							   &pPrimaryGroupdsid,					// primary group SID
//							   &pdacl,								// DACL
//							   NULL,								// SACL
//							   &pSD);								// Security Descriptor
//
//	if (res != ERROR_SUCCESS) {
//		if (res == ERROR_FILE_NOT_FOUND) {
//			// should never get here. before calling this function the file should already have 
//			// been checked for existance.
//			throw ProbeException("Error unable locate " + filePath + " while getting trustte names."); 
//		} else {
//			throw ProbeException("Error unable to retrieve a copy of the security descriptor for " + filePath + " while getting trustee names."); 
//		}
//	} 
//
//	// Check to see if a valid security descriptor were returned.  
//    if ((IsValidSecurityDescriptor(pSD) == 0) || (IsValidAcl(pdacl) == 0)  || (IsValidSid(pOwnersid) == 0) || (IsValidSid(pPrimaryGroupdsid) == 0)) {
//		LocalFree(pSD);
//		throw ProbeException("Error invalid data returned from call to GetNamedSecurityInfo(). Cannot get effective rights for file: " + filePath);
//	}
//
//	// process dacl
//	// get effecitve rights from acl
//	// http://windowssdk.msdn.microsoft.com/en-us/library/ms719346.aspx
//
//	// need to get sid string
//	LPTSTR sidString;
//	Common::GetTextualSid(psid, &sidString);
//	string trusteeSid = sidString;
//	free(sidString);
//
//	// init the Trustee structure
//	TRUSTEE trustee;
//	trustee.pMultipleTrustee  = NULL;
//	trustee.MultipleTrusteeOperation = NO_MULTIPLE_TRUSTEE;
//	trustee.TrusteeForm = TRUSTEE_IS_NAME;
//	//trustee.TrusteeForm = TRUSTEE_IS_SID;
//	trustee.TrusteeType = TRUSTEE_IS_UNKNOWN;
//	//trustee.ptstrName = psid;
//	trustee.ptstrName = const_cast<char*>(trusteeSid.c_str());
//
//	ACCESS_MASK accessRights;
//	res = GetEffectiveRightsFromAcl(pdacl,
//									&trustee,
//									&accessRights);
//	if (res != ERROR_SUCCESS) {
//		LocalFree(pSD);
//		throw ProbeException("Error unable to get effictive rights for trustee from dacl for file: " + filePath); 
//	}
//		
//	// Convert access mask to binary.
//	int j;
//	char mask[33];
//
//	for (j = 0; j < 32; j++) {
//		if (accessRights & (1 << j))
//			mask[31 - j] = '1';
//		else
//			mask[31 - j] = '0';
//	}
//	mask[32] = 0;
//
//	item = this->CreateItem();
//	item->SetStatus(OvalEnum::EXISTS_STATUS);
//	item->AppendElement(new ItemEntity("path", path, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("filename", fileName, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("trustee_name", "", OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("trustee_domain", "", OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//
//	item->AppendElement(new ItemEntity("trustee_sid", trusteeSid, OvalEnum::STRING_TYPE, true, OvalEnum::EXISTS_STATUS));
//
//	// read values in the access_mask
//	// http://windowssdk.msdn.microsoft.com/en-us/library/ms717896.aspx
//	item->AppendElement(new ItemEntity("standard_delete", Common::ToString(mask[16]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_read_control", Common::ToString(mask[17]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_write_dac", Common::ToString(mask[18]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_write_owner", Common::ToString(mask[19]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("standard_synchronize", Common::ToString(mask[20]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("access_system_security", Common::ToString(mask[24]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_read", Common::ToString(mask[31]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_write", Common::ToString(mask[30]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_execute", Common::ToString(mask[29]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("generic_all", Common::ToString(mask[28]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_data", Common::ToString(mask[0]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_data", Common::ToString(mask[1]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_append_data", Common::ToString(mask[2]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_ea", Common::ToString(mask[3]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_ea", Common::ToString(mask[4]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_execute", Common::ToString(mask[5]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_delete_child", Common::ToString(mask[6]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_read_attributes", Common::ToString(mask[7]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//	item->AppendElement(new ItemEntity("file_write_attributes", Common::ToString(mask[8]), OvalEnum::STRING_TYPE, false, OvalEnum::EXISTS_STATUS));
//
//	LocalFree(pSD);
//
//	return item;
//}
